# -*- coding: utf-8 -*-

"""Main module."""

import re

from collections import namedtuple
from pathlib import Path

# on the complete regex '.*' is not used, since it would also capture
# double column definitions like "something_AB12"
REGEX_ID = r"(?P<row>[A-H])(?P<col>(0?[1-9]|1[0-2]))_(?P<channel>\d)"
REGEX_SENSOVATION = r"(?P<stem>(.+_|))" + REGEX_ID
REGEX_SCIENION = REGEX_ID + r"(?P<stem>(_.+|))"

RE_SENSOVATION = re.compile(REGEX_SENSOVATION)
RE_SCIENION = re.compile(REGEX_SCIENION)

TPL_ID = "{row}{col:0>{dec}d}_{channel}"
TPL_SENSOVATION = "{stem}" + TPL_ID
TPL_SCIENION = TPL_ID + "{stem}"


Renamable = namedtuple("Renamable", "path,stem,row,col,channel,ext")
FileNameTemplate = namedtuple("FileNameTemplate", "template,prefix,suffix")

template_sensovation = FileNameTemplate(template=TPL_SENSOVATION, prefix="", suffix="_")
template_scienion = FileNameTemplate(TPL_SCIENION, prefix="_", suffix="")


class S2IOError(IOError):
    """ custom IO Exception """


def get_source_list(folder, suffix=None):
    """ returns the files in a directory that are not hidden

    if 'suffix' is given, the list will only return files with the given
    extension
    """
    src_dir = Path(folder)
    if not src_dir.is_dir():
        raise S2IOError(f"{src_dir} is not a directory")
    all_files = (f for f in src_dir.iterdir() if f.is_file())
    non_hidden_files = (f for f in all_files if not f.stem.startswith("."))
    if suffix is None:
        return non_hidden_files
    else:
        return filter_source_list(non_hidden_files, suffix)


def filter_source_list(source_list, suffix):
    """ filters pathlib.Path objects by only containing a given suffix """
    return (f for f in source_list if f.suffix.lower() == suffix)


def parse_source_list(source_list, regex):
    """ returns a list of items that can be renamed """
    for path in source_list:
        if path.name.startswith("."):
            continue
        groups = regex.match(path.stem)
        if groups:
            stem = groups["stem"].strip("_")
            entry = Renamable(
                path=path,
                stem=stem,
                row=groups["row"],
                col=int(groups["col"]),
                channel=groups["channel"],
                ext=path.suffix,
            )
            yield entry


def get_new_name(item, name_template, digits):
    stem = item.stem
    if stem:
        # only apply prefix and suffix if the name is not empty
        stem = name_template.prefix + stem + name_template.suffix
    new_name = name_template.template.format(
        stem=stem, row=item.row, col=item.col, channel=item.channel, dec=digits
    )
    return new_name + item.ext


def rename_files(renamable_list, name_template, digits):
    for item in renamable_list:
        new_name = get_new_name(item, name_template, digits)
        new_path = item.path.with_name(new_name)
        item.path.replace(new_path)
